import 'dart:math';

import 'package:flutter/material.dart';

import '../entities/cell.dart';
import '../layout/board.dart';
import '../provider/data.dart';
import '../utils/board_animate.dart';
import '../utils/board_utils.dart';
import '../utils/game_utils.dart';

class Game {

  static Container buildGameWidget(Data myProvider) {
    bool gameIsFinished = BoardUtils.checkBoardIsSolved(myProvider);
    if (gameIsFinished) {
      BoardAnimate.startAnimation(myProvider, 'win');
    }

    return Container(
      child: Column(
        mainAxisAlignment: MainAxisAlignment.start,
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          Board.buildGameBoard(myProvider),
          SizedBox(height: 2),
          gameIsFinished ? Game.buildWinMessage(myProvider) : Game.buildSelectCellValueBar(myProvider),
        ],
      ),
    );
  }


  static Container buildSelectCellValueBar(Data myProvider) {
    List cells = myProvider.cells;

    Color borderColor = Colors.blue;

    bool isCellSelected = (myProvider.currentCellCol != null && myProvider.currentCellRow != null);
    bool isUpdatableCellSelected = isCellSelected ? !myProvider.cells[myProvider.currentCellRow][myProvider.currentCellCol].isFixed : false;
    int maxValue = myProvider.blockSizeHorizontal * myProvider.blockSizeVertical;

    int maxItemsPerLine = 10;
    int linesCount = (maxValue / maxItemsPerLine).floor() + 1;
    int itemsCountPerLine = min(maxItemsPerLine, maxValue + 1);

    return Container(
      margin: EdgeInsets.all(2),
      padding: EdgeInsets.all(2),

      child: Table(
        defaultColumnWidth: IntrinsicColumnWidth(),
        children: [
          for (var lineIndex = 0; lineIndex < linesCount; lineIndex++)
            TableRow(
              children: [
                for (
                  var value = lineIndex * itemsCountPerLine;
                  value < (lineIndex + 1) * itemsCountPerLine;
                  value++
                )
                  Column(
                    children: [
                      Cell(
                        isUpdatableCellSelected ? (value <= maxValue ? value : -1) : -1,
                        false
                      ).widgetUpdateValue(myProvider)
                    ]
                  ),
              ]
            ),
        ]
      ),
    );
  }


  static Container buildWinMessage(Data myProvider) {
    Column decorationImage = Column(
      children: [
        Image(
          image: AssetImage('assets/icons/game_win.png'),
          fit: BoxFit.fill
        ),
      ]
    );

    return Container(
      margin: EdgeInsets.all(2),
      padding: EdgeInsets.all(2),

      child: Table(
        defaultColumnWidth: IntrinsicColumnWidth(),
        children: [
          TableRow(
            children: [
              decorationImage,
              Column(
                children: [
                  FlatButton(
                    child: Container(
                      child: Image(
                        image: AssetImage('assets/icons/button_back.png'),
                        fit: BoxFit.fill
                      ),
                    ),
                    onPressed: () => GameUtils.resetGame(myProvider),
                  ),
                ]
              ),
              decorationImage,
            ],
          ),
        ]
      )
    );
  }

}
